@@ -204,15 +204,15 @@ lav_bvord_cor_twostep_fit <- function(Y1, Y2, eXo = NULL, wt = NULL,
             warning("lavaan WARNING: ",
                     "estimation polychoric correlation did not converge for
                     variables ", Y1.name, " and ", Y2.name)
-         } else {
-             warning("lavaan WARNING: estimation polychoric correlation(s)",
-                     " did not always converge")
-         }
+        } else {
+            warning("lavaan WARNING: estimation polychoric correlation(s)",
+                    " did not always converge")
+        }
+        rho <- start.x
+    } else {
+        rho <- optim$par
     }
 
-    # store result
-    rho <- optim$par
-
     # zero.cell.flag
     if(zero.cell.flag) {
         attr(rho, "zero.cell.flag") <- empty.cells
